"""Custom events dispatched by the canvas.

These events may later be used within a plugin system, where plugins are allowed to bind their own
handlers to these events.
"""
# from __future__ import annotations
# pylint: disable=maybe-no-member
from collections import defaultdict
from dataclasses import dataclass, fields, is_dataclass
from typing import (
    Any,
    Callable,
    DefaultDict,
    Dict,
    List,
    Optional,
    Set,
    Tuple,
    Type, Union,
)

import wx

from rkviewer.canvas.geometry import Vec2

# ------------------------------------------------------------
from inspect import getframeinfo, stack
# ------------------------------------------------------------

class CanvasEvent:
    def to_tuple(self):
        assert is_dataclass(self), "as_tuple requires the CanvasEvent instance to be a dataclass!"

        return tuple(getattr(self, f.name) for f in fields(self))


@dataclass
class SelectionDidUpdateEvent(CanvasEvent):
    """Called after the list of selected nodes and/or reactions has changed.

    Attributes:
        node_indices: The indices of the list of selected nodes.
        reaction_indices: The indices of the list of selected reactions.
        compartment_indices: The indices of the list of selected compartments.
    """
    node_indices: Set[int]
    reaction_indices: Set[int]
    compartment_indices: Set[int]


@dataclass
class CanvasDidUpdateEvent(CanvasEvent):
    """Called after the canvas has been updated by the controller."""
    pass

@dataclass
class DidNewNetworkEvent(CanvasEvent):
    """ Called when the canvas is cleared by choosing "New".
    """
    pass

@dataclass
class DidMoveNodesEvent(CanvasEvent):
    """Called after the position of a node changes but has not been committed to the model.

    This event may be called many times, continuously as the user drags a group of nodes. Note that
    only after the drag operation has ended, is model notified of the move for undo purposes. See
    DidCommitDragEvent.

    Attributes:
        node_indices: The indices of the nodes that were moved.
        offset: The position offset. If all nodes were moved by the same offset, then a single Vec2
                is given; otherwise, a list of offsets are given, with each offset matching a node.
        dragged: Whether the resize operation was done by the user dragging, and not, for exmaple,
                 through the form.
        by_user: Whether the event was performed by the user or through a plugin.
    """
    node_indices: List[int]
    offset: Union[Vec2, List[Vec2]]
    dragged: bool
    by_user: bool = True


@dataclass
class DidMoveCompartmentsEvent(CanvasEvent):
    """
    Same as `DidMoveNodesEvent` but for compartments.

    Attributes:
        compartment_indices: The indices of the compartments that were moved.
        offset: The position offset. If all compartments were moved by the same offset,
                then a single Vec2 is given; otherwise, a list of offsets are given,
                with each offset matching a node.
        dragged: Whether the resize operation was done by the user dragging, and not, for example,
                 through the form.
        by_user: Whether the event was performed by the user or through a plugin.
    """
    compartment_indices: List[int]
    offset: Union[Vec2, List[Vec2]]
    dragged: bool
    by_user: bool = True


@dataclass
class DidResizeNodesEvent(CanvasEvent):
    """Called after the list of selected nodes has been resized.

    Attributes:
        node_indices: The indices of the list of resized nodes.
        ratio: The resize ratio.
        dragged: Whether the resize operation was done by the user dragging, and not, for exmaple,
                 through the form.
        by_user: Whether the event was performed by the user or through a plugin.
    """
    node_indices: List[int]
    ratio: Vec2
    dragged: bool
    by_user: bool = True


@dataclass
class DidResizeCompartmentsEvent(CanvasEvent):
    """Called after the list of selected compartments has been resized.

    Attributes:
        compartment_indices: The indices of the list of resized compartments.
        ratio: The resize ratio.
        dragged: Whether the resize operation was done by the user dragging, and not, for exmaple,
                 through the form.
        by_user: Whether the event was performed by the user or through a plugin.
    """
    compartment_indices: List[int]
    ratio: Union[Vec2, List[Vec2]]
    dragged: bool
    by_user: bool = True


@dataclass
class DidCommitDragEvent(CanvasEvent):
    """Dispatched after any continuously emitted dragging event has concluded.

    This is dispatched for any event that is posted in quick intervals while the mouse left
    button is held while moving, i.e. "dragging" events. This includes: DidMoveNodesEvent,
    DidMoveCompartmentsEvent, DidResizeNodesEvent, DidResizeCompartmentsEvent,
    and DidResizeMoveBezierHandlesEvent. This event is emitted after the left mouse button is
    released, the model is notified of the change, and the action is complete.
    """
    source: Any


@dataclass
class DidMoveBezierHandleEvent(CanvasEvent):
    """Dispatched after a Bezier handle is moved.

    Attributes:
        net_index: The network index.
        reaction_index: The reaction index.
        node_index: The index of the node whose Bezier handle moved. -1 if the source centroid
                    handle was moved, or -2 if the dest centroid handle was moved.
        direct: Automatically true when by_user is False. Otherwise, True if the handle is
                moved by the user dragging the handle directly, and False if the handle was moved
                by the user dragging the node associated with that handle.
        by_user: Whether the event was performed by the user or through a plugin.
    """
    net_index: int
    reaction_index: int
    node_index: int
    by_user: bool
    direct: bool


@dataclass
class DidMoveReactionCenterEvent(CanvasEvent):
    """Dispatched after the reaction center is moved by the user.

    Note that this is not triggered if the center moved automatically due to nodes moving.

    Attributes:
        net_index: The network index.
        reaction_index: The reaction index.
        offset: The amount moved.
        dragged: Whether the center is moved by the user dragging (it could have been through the
                  form).
    """
    net_index: int
    reaction_index: int
    offset: Vec2
    dragged: bool


@dataclass
class DidAddNodeEvent(CanvasEvent):
    """Called after a node has been added.

    Attributes:
        node: The index of the node that was added.

    Note:
        This event triggers only if the user has performed a drag operation, and not, for example,
        if the user moved a node in the edit panel.
    TODO in the documentation that this event and related ones (and DidDelete-) are emitted before
    controller.end_group() is called. As an alternative, maybe create a call_after() function
    similar to wxPython? it should be called in OnIdle() or Refresh()
    """
    node: int


@dataclass
class DidDeleteEvent(CanvasEvent):
    """Called after a node has been deleted.

    Attributes:
        node_indices: The set of nodes (indices )that were deleted.
        reaction_indices: The set of reactions (indices) that were deleted.
        compartment_indices: The set of compartment (indices) that were deleted.
    """
    node_indices: Set[int]
    reaction_indices: Set[int]
    compartment_indices: Set[int]


@dataclass
class DidAddReactionEvent(CanvasEvent):
    """Called after a reaction has been added.

    Attributes:
        reaction: The Reaction that was added.
    """
    index: int
    sources: List[int]
    targets: List[int]


@dataclass
class DidAddCompartmentEvent(CanvasEvent):
    """Called after a compartment has been added.

    Attributes:
        compartment: The Compartment that was added.
    """
    index: int


@dataclass
class DidChangeCompartmentOfNodesEvent(CanvasEvent):
    """Called after one or more nodes have been moved to a new compartment.

    Attributes:
        node_indices: The list of node indices that changed compartment.
        old_compi: The old compartment index, -1 for base compartment.
        new_compi: The new compartment index, -1 for base compartment.
        by_user: Whether this event was triggered directly by a user action, as opposed to by a
                 plugin.
    """
    node_indices: List[int]
    old_compi: int
    new_compi: int
    by_user: bool = True


@dataclass
class DidModifyNodesEvent(CanvasEvent):
    """Called after a property of one or more nodes has been modified, excluding position or size.

    For position and size events, see DidMove...Event() and DidResize...Event()

    Attributes:
        nodes: The indices of the list of nodes that were modified.
        by_user: Whether this event was triggered directly by a user action and not, for example,
                 by a plugin.
    """
    indices: List[int]
    by_user: bool = True


@dataclass
class DidModifyReactionEvent(CanvasEvent):
    """Called after a property of one or more nodes has been modified, excluding position.

    Attributes:
        indices: The indices of the list of reactions that were modified.
        by_user: Whether this event was triggered directly by a user action and not, for example,
                 by a plugin.
    """
    indices: List[int]
    by_user: bool = True


@dataclass
class DidModifyCompartmentsEvent(CanvasEvent):
    """Called after a property of one or more compartments has been modified, excluding position or size.

    For position and size events, see DidMove...Event() and DidResize...Event()

    Attributes:
        indices: The indices of list of compartments that were modified.
    """
    indices: List[int]


@dataclass
class DidUndoEvent(CanvasEvent):
    """Called after an undo action is done."""
    by_user: bool = True


@dataclass
class DidRedoEvent(CanvasEvent):
    """Called after a redo action is done."""
    by_user: bool = True


@dataclass
class DidPaintCanvasEvent(CanvasEvent):
    """Called after the canvas has been painted.

    Attributes:
        gc: The graphics context of the canvas.
    """
    gc: wx.GraphicsContext


EventCallback = Callable[[CanvasEvent], None]
class HandlerNode:
    next_: Optional['HandlerNode']
    prev: Optional['HandlerNode']
    handler: EventCallback

    def __init__(self, handler: EventCallback):
        self.handler = handler
        self.next_ = None


# Maps CanvasElement to a dict that maps events to handler nodes
handler_map: Dict[int, Tuple['HandlerChain', HandlerNode]] = dict()
# Maps event to a chain of handlers
event_chains: DefaultDict[Type[CanvasEvent], 'HandlerChain'] = defaultdict(lambda: HandlerChain())

handler_id = 0


class HandlerChain:
    head: Optional[HandlerNode]
    tail: Optional[HandlerNode]

    def __init__(self):
        self.head = None
        self.tail = None
        self.it_cur = None

    def remove(self, node: HandlerNode):
        if node.prev is not None:
            node.prev.next_ = node.next_
        else:
            self.head = node.next_

        if node.next_ is not None:
            node.next_.prev = node.prev
        else:
            self.tail = node.prev

    def __iter__(self):
        self.it_cur = self.head
        return self

    def __next__(self):
        if self.it_cur is None:
            raise StopIteration()

        ret = self.it_cur.handler
        self.it_cur = self.it_cur.next_
        return ret

    def append(self, handler: EventCallback) -> HandlerNode:
        node = HandlerNode(handler)
        if self.head is None:
            assert self.tail is None
            self.head = self.tail = node
        else:
            assert self.tail is not None
            node.prev = self.tail
            self.tail.next_ = node
            self.tail = node

        return node


def bind_handler(evt_cls: Type[CanvasEvent], callback: EventCallback) -> int:
    global handler_id
    ret = handler_id
    chain = event_chains[evt_cls]
    hnode = chain.append(callback)
    handler_map[ret] = (chain, hnode)
    handler_id += 1
    return ret


def unbind_handler(handler_id: int):
    chain, hnode = handler_map[handler_id]
    chain.remove(hnode)
    del handler_map[handler_id]


def post_event(evt: CanvasEvent):
    '''
    # debugging
    if not str(evt)[:14]=="DidPaintCanvas":
        caller = getframeinfo(stack()[1][0])
        print("%s:%d - %s" % (caller.filename, caller.lineno, str(evt)))
    '''

    for callback in iter(event_chains[type(evt)]):
        callback(evt)



"""
Do the structural analysis.
Version 1.0.0: Author: Jin Xu (2020)
"""


# pylint: disable=maybe-no-member
import wx
import wx.grid as  gridlib
from rkviewer.plugin.classes import PluginMetadata, WindowedPlugin, PluginCategory
from rkviewer.plugin import api
from rkviewer.plugin.api import Node, Vec2, Reaction, Color
import math
import random as _random
import numpy as _np
import copy as _copy
from dataclasses import dataclass

class TabOne(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, wx.EXPAND|wx.ALL)
        self.grid_st = gridlib.Grid(self)
        self.grid_st.CreateGrid(20,20)
        for i in range(20):
            self.grid_st.SetColLabelValue(i, "")
        for i in range(20):
            self.grid_st.SetRowLabelValue(i, "")

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.grid_st, 1, wx.EXPAND)
        self.SetSizer(sizer)

class TabTwo(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, wx.EXPAND|wx.ALL)
        self.grid_moi = gridlib.Grid(self)
        self.grid_moi.CreateGrid(20,20)
        for i in range(20):
            self.grid_moi.SetColLabelValue(i, "")
        for i in range(20): 
            self.grid_moi.SetRowLabelValue(i, "")

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.grid_moi, 1, wx.EXPAND)
        self.SetSizer(sizer)

class StructuralAnalysis(WindowedPlugin):
    metadata = PluginMetadata(
        name='StructuralAnalysis',
        author='Jin Xu',
        version='1.0.0',
        short_desc='Structural Analysis.',
        long_desc='StructuralAnalysis Plugin is to calculate and visualize the stoichiometry matrix and conserved moieties for the network.',
        category=PluginCategory.ANALYSIS
    )
    def __init__(self):
        """
        Initialize the StructuralAnalysis Plugin.
        Args:
            self
        """
        super().__init__()
        self.index_list=[]


    def create_window(self, dialog):
        """
        Create a window to do the structural analysis.
        Args:
            self
            dialog
        """
        topPanel = wx.Panel(dialog, pos=(0,0), size=(700, 500))

        # Create two panels size by side
        panel1 = wx.Panel(topPanel, -1, pos=(0,100), size=(200,100))
        panel2 = wx.Panel(topPanel, -1, pos=(100,100),size=(450,100))

        sizer = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(panel1,0,wx.EXPAND|wx.ALL,border=10)
        sizer.Add(panel2,0,wx.EXPAND|wx.ALL,border=10)
        
        topPanel.SetSizer(sizer)

        # Add a notebook to the second panel
        nb = wx.Notebook(panel2)

        # Create the tabs
        self.tab1 = TabOne(nb)
        self.tab2 = TabTwo(nb)
        nb.AddPage(self.tab1, "Stoichiometry Matrix")
        nb.AddPage(self.tab2, "Conservation Matrix")

        # Make sure the second panel fills the right side. 
        sizer = wx.BoxSizer()
        sizer.Add(nb, 1, wx.EXPAND)
        panel2.SetSizer(sizer)
      
        Compute_btn = wx.Button(panel1, -1, 'Compute Matrix', (20,20))
        Compute_btn.Bind(wx.EVT_BUTTON, self.Compute)

        wx.StaticText(panel1, -1, 'Select a row from the table of', (20,100))
        wx.StaticText(panel1, -1, 'Moiety Conservation Laws', (20,120))
        wx.StaticText(panel1, -1, 'and pick a color:', (20,140))

        Picker = wx.ColourPickerCtrl(panel1, pos=(20,160))
        Picker.Bind(wx.EVT_COLOURPICKER_CHANGED, self.color_callback)

        wx.StaticText(panel1, -1, 'Unhighlight the nodes:', (20,240))
        Clear_Btn = wx.Button(panel1, -1, 'Clear', (20,260))
        Clear_Btn.Bind(wx.EVT_BUTTON, self.unhighlight)

        return topPanel

    def on_did_create_dialog(self):
        # Set position of popup window to top-left corner of screen
        self.dialog.SetPosition((240, 250))


    def Compute(self, evt):
        """
        Handler for the "Compute" button.
        Get the network on canvas.
        Calculate the Stoichiometry Matrix and Conservation Matrix for the randon network.
        """
        def nullspace(A, atol=1e-13, rtol=0):  
            A = _np.atleast_2d(A)
            u, s, vh = _np.linalg.svd(A)
            tol = max(atol, rtol * s[0])
            nnz = (s >= tol).sum()
            ns = vh[nnz:].conj().T
            return ns

        #https://gist.github.com/sgsfak/77a1c08ac8a9b0af77393b24e44c9547
        def rref(B, tol=1e-8, debug=False):
          A = B.copy()
          rows, cols = A.shape
          r = 0
          pivots_pos = []
          row_exchanges = _np.arange(rows)
          for c in range(cols):
            if debug: print ("Now at row", r, "and col", c, "with matrix:"); print (A)
        
            ## Find the pivot row:
            pivot = _np.argmax (_np.abs (A[r:rows,c])) + r
            m = _np.abs(A[pivot, c])
            if debug: print ("Found pivot", m, "in row", pivot)
            if m <= tol:
              ## Skip column c, making sure the approximately zero terms are
              ## actually zero.
              A[r:rows, c] = _np.zeros(rows-r)
              if debug: print ("All elements at and below (", r, ",", c, ") are zero.. moving on..")
            else:
              ## keep track of bound variables
              pivots_pos.append((r,c))
        
              if pivot != r:
                ## Swap current row and pivot row
                A[[pivot, r], c:cols] = A[[r, pivot], c:cols]
                row_exchanges[[pivot,r]] = row_exchanges[[r,pivot]]
                
                if debug: print ("Swap row", r, "with row", pivot, "Now:"); print (A)
        
              ## Normalize pivot row
              A[r, c:cols] = A[r, c:cols] / A[r, c];
        
              ## Eliminate the current column
              v = A[r, c:cols]
              ## Above (before row r):
              if r > 0:
                ridx_above = _np.arange(r)
                A[ridx_above, c:cols] = A[ridx_above, c:cols] - _np.outer(v, A[ridx_above, c]).T
                if debug: print ("Elimination above performed:"); print (A)
              ## Below (after row r):
              if r < rows-1:
                ridx_below = _np.arange(r+1,rows)
                A[ridx_below, c:cols] = A[ridx_below, c:cols] - _np.outer(v, A[ridx_below, c]).T
                if debug: print ("Elimination below performed:"); print (A)
              r += 1
            ## Check if done
            if r == rows:
              break;
          return (A, pivots_pos, row_exchanges)


        netIn = 0
        numNodes = api.node_count(netIn)
        
        if numNodes == 0:
            wx.MessageBox("Please import a network on canvas", "Message", wx.OK | wx.ICON_INFORMATION)
        else:
            allNodes = api.get_nodes(netIn)
            node =  allNodes[0]
            try:
                primitive, transform = node.shape.items[0]
                self.default_color = primitive.fill_color
            except:
                self.default_color = api.Color(255, 204, 153) #random network node color

            largest_node_index = 0
            for i in range(numNodes):
                if allNodes[i].index > largest_node_index:
                    largest_node_index = allNodes[i].index
            row = largest_node_index + 1
            numReactions = api.reaction_count(netIn)
            col = numReactions
            self.st = _np.zeros((row, col))
            allReactions = api.get_reactions(netIn)
            for i in range(numReactions):
                for j in range(len(allReactions[i].sources)):
                    for m in range(row):
                        if allReactions[i].sources[j] == m:
                            self.st.itemset((m, i), -1)
                for j in range(len(allReactions[i].targets)):
                    for m in range(row):
                        if allReactions[i].targets[j] == m:
                            self.st.itemset((m,i), 1)


            stt = _np.transpose (self.st)
            m = _np.transpose (nullspace (stt))
            moi_mat = rref(m)[0]

            # set all the values of non-existing nodes to zero
            for i in range(moi_mat.shape[0]):
                for j in range(moi_mat.shape[1]):
                    if _np.array_equal(self.st[j,:], _np.zeros(self.st.shape[1])):
                        moi_mat.itemset((i,j), 0.)

            for i in range(self.st.shape[1]):
                self.tab1.grid_st.SetColLabelValue(i, "J" + str(i))
            for i in range(self.st.shape[0]):
                id = allNodes[i].id
                self.tab1.grid_st.SetRowLabelValue(i, id)
            
            for row in range(self.st.shape[0]):
                for col in range(self.st.shape[1]):
                    self.tab1.grid_st.SetCellValue(row, col,"%d" % self.st.item(row,col))

            for i in range(moi_mat.shape[1]):
                id = allNodes[i].id
                self.tab2.grid_moi.SetColLabelValue(i, id)

            CSUM_id = 0

            for i in range(moi_mat.shape[0]):
                a = moi_mat[i,:]
                a = [0. if abs(a_) < 0.005 else a_ for a_ in a] # some elements are very small
                if _np.array_equal(a, _np.zeros(moi_mat.shape[1])): # delete the row if all the elements are zero
                    CSUM_id = CSUM_id
                else:
                    self.tab2.grid_moi.SetRowLabelValue(CSUM_id, "CSUM" + str(CSUM_id))    

                    for j in range(moi_mat.shape[1]):
                        self.tab2.grid_moi.SetCellValue(CSUM_id, j, format (a[j], ".2f")) 
                    CSUM_id += 1 


    def printSelectedCells(self, top_left, bottom_right):
        """
        Based on code from http://ginstrom.com/scribbles/2008/09/07/getting-the-selected-cells-from-a-wxpython-grid/
        """
        cells = []
        
        rows_start = top_left[0]
        rows_end = bottom_right[0]
        cols_start = top_left[1]
        cols_end = bottom_right[1]
        rows = range(rows_start, rows_end+1)
        cols = range(cols_start, cols_end+1)
        cells.extend([(row, col)
            for row in rows
            for col in cols])
            
        self.index_list = []
        for cell in cells:
            row, col = cell
            value = self.tab2.grid_moi.GetCellValue(row,col)
            if value != "0.00" and value != "+0.00" and value != "-0.00" and value !="":
                self.index_list.append(int(col))


    def color_callback(self, evt):
                   
        """
        Get whatever cells are currently selected
        """

        cells = self.tab2.grid_moi.GetSelectedCells()
        if not cells:
            if self.tab2.grid_moi.GetSelectionBlockTopLeft():
                top_left = self.tab2.grid_moi.GetSelectionBlockTopLeft()[0]
                bottom_right = self.tab2.grid_moi.GetSelectionBlockBottomRight()[0]
                self.printSelectedCells(top_left, bottom_right)
            #else:
            #    print (self.currentlySelectedCell)
        else:
            print("no cells are selected")

        
        """
        Callback for the color picker control; sets the color of every node/reaction selected.
        """

        wxcolor = evt.GetColour()
        color = Color.from_rgb(wxcolor.GetRGB())

        # start group action context for undo purposes
        with api.group_action():
            # color selected nodes
            #for index in api.selected_node_indices():

            if len(self.index_list) == 0:
                wx.MessageBox("Please select a row and pick a color again", "Message", wx.OK | wx.ICON_INFORMATION)
            try:
                for index in self.index_list:
                    api.update_node(api.cur_net_index(), index, fill_color=color)
            except:
                wx.MessageBox("Please select a row and pick a color again", "Message", wx.OK | wx.ICON_INFORMATION)


    def unhighlight(self, evt):

        """
        Callback for the color picker control; sets the color of every node/reaction selected.
        """
        
        # start group action context for undo purposes
        with api.group_action():
            # color selected nodes
            #for index in api.selected_node_indices():
            try:
                for index in self.index_list:
                    api.update_node(api.cur_net_index(), index, fill_color=self.default_color)
            except:
                wx.MessageBox("There is no highlighted nodes", "Message", wx.OK | wx.ICON_INFORMATION)





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rkviewer',
 'rkviewer.canvas',
 'rkviewer.plugin',
 'rkviewer.resources',
 'rkviewer_plugins']

package_data = \
{'': ['*'], 'rkviewer': ['json/*']}

install_requires = \
['commentjson>=0.9.0,<0.10.0',
 'marshmallow-polyfield>=5.10,<6.0',
 'marshmallow>=3.11.1,<4.0.0',
 'networkx>=2.5.1,<3.0.0',
 'pandas>=1.3.0,<2.0.0',
 'python-libsbml>=5.18.0,<6.0.0',
 'requests',
 'simplesbml>=2.2.0,<3.0.0',
 'sortedcontainers>=2.3.0,<3.0.0',
 'traitlets',
 'wheel',
 'wxPython>=4.1.1,<5.0.0']

extras_require = \
{':python_version >= "3.6" and python_version < "3.7"': ['dataclasses>=0.8,<0.9'],
 'simulation': ['tellurium>=2.2.1,<3.0.0']}

entry_points = \
{'console_scripts': ['SBcoyote = rkviewer.main:main']}

setup_kwargs = {
    'name': 'sbcoyote',
    'version': '1.2.8',
    'description': 'SBcoyote: An Extensible Python Based Reaction Editor and Viewer.',
    'long_description': '\n# SBcoyote: An Extensible Python Based Reaction Editor and Viewer.\n\n## Introduction\n\nSBcoyote, initially called PyRKViewer, is a cross-platform visualization tool for drawing reaction networks written with the\n[wxPython](https://www.wxpython.org/) framework. It can draw reactants, products, reactions, and compartments, and its features include but are not limited to:\n* Support for floating and boundary species.\n* Reactions can be displayed using Bezier curves and straight lines.\n* Plugin support, with some plugin examples: Arrow Designer, Random Network, Auto Layout, etc.\n\n## Installment for Users\n\n* Install Python 3.7, 3.8, 3.9 or 3.10 if not ready in the system.\n* Go to the command line and type `pip install SBcoyote`.\n* If wxPython won\'t get installed automatically, please try to install wxPython 4.1.1 or 4.2.0 manually referring to https://wxpython.org/pages/downloads/index.html. Note wxPython 4.1.1 does not apply to Python 3.10. \n* To run the application, simply type in the command line `SBcoyote`.\n\n## Documentation\n\nThe full documentation can be found at: https://sys-bio.github.io/SBcoyote/\n\n## Installment options for Developers\n\n### Installing with Poetry\n1. If you do not have poetry installed on your computer, follow the quick steps shown [here](https://python-poetry.org/docs/).\n2. Once you have poetry installed, you will download SBcoyote. Click the green button at the top of this page that says “Code” and choose “Download ZIP”. You want to make sure you know where you have downloaded this. Unzip the folder to your desired directory.\n3. Next, open your terminal and navigate to the directory containing SBcoyote.\n4. Once inside the main folder of the application you can install the dependencies. To install the base dependencies simply run `poetry install`. To install the optional ones as well, run `poetry install -E simulation`. Note that this step may take a while. To learn more about which set of dependencies is right for you, refer to the [Dependencies](#Dependencies) section below.\n5. Finally, you will run the application with the command `poetry run SBcoyote`.\n\nAfter you have completed all of these steps, you will not have to repeat them every time you want to run the application. Once the setup is done you will only need to open the terminal, navigate into the folder that contains your SBcoyote application, and run the command `poetry run SBcoyote`.\n\n### Installing without Poetry\nAgain, we strongly advise following the steps above, as it makes the set-up process much faster and simpler. However, to install SBcoyote without Poetry, here is the process you will follow:\n\n1. First, download SBcoyote. Click the green button at the top of this page that says “Code” and choose “Download ZIP”. You want to make sure you know where you have downloaded this. Unzip the folder to your desired directory.\n2. Next, open your terminal and navigate to the directory containing SBcoyote.\n3. To install the base set of dependencies, you will run `pip install -r requirements.txt`. Then if you want to install the optional dependencies as well, run `pip install -r requirements-simulation.txt`. To learn more about which set of dependencies is right for you, refer to the [Dependencies](#Dependencies) section below.\n4. Finally, you will run the application with the command `python -m rkviewer.main`.\nAfter you have completed all of these steps, you will not have to repeat them every time you want to run the application. Once the setup is done you will only need to open the terminal, navigate into the folder that contains your SBcoyote application, and run the command `python -m rkviewer.main`.\n\n### Running\n* If you have poetry, simply run `poetry run SBcoyote`.\n* Otherwise, in your virtual environment, run `python -m rkviewer.main`.\n* Then, check out the [documentation](#documentation).\n\n## Development Setup\n\n### Dependencies\nWe are using [poetry](https://python-poetry.org/) for dependency management. If you are just looking\nto build and run, though, you can work solely with `pip` as well.\n\nThere are currently three dependency groups: "base", "development", and "simulation".\n* "base" is the bare minimum requirements to run the application without any plugins.\n* "development" includes the additional requirements for development, such as for documentation\nand testing.\n* "simulation" includes a large set of dependencies required for running simulation related Plugins. (This is in addition to the base requirements).\n\nThe dependency groups are specified in `pyproject.toml` for `poetry`. There are additionally\n`requirement.txt` files generated by `poetry`, including `requirements.txt`, `requirements-dev.txt`,\nand `requirements-simulation.txt`. If you do not have poetry, you can opt for those as well. If you are\nusing linux, extra work would need to be done on installing wxPython. Please refer to the\n"Linux Notes" section below.\n\n### Installing Dependencies\n`poetry` is recommended for installing dependencies. Simply `poetry install` for the base\ndependencies and `poetry install -E simulation` to install the optional ones as well.\n\nIf you don\'t have poetry, you can simply run `pip install -r <>` for any of the aforementioned\n`requirements.txt` files.\n\n### Running locally\n* If you have poetry, simply `poetry run SBcoyote`.\n* Otherwise, in your virtual environment, run `python -m rkviewer.main`.\n\n## Development Distributing\n\n* Use `poetry build` and `poetry publish`. Refer to [poetry docs](https://python-poetry.org/docs/)\nfor more detail.\n* To re-generate the `requirements*.txt`, run `scripts/gen_requirements.py`.\n\n### Bundling an Executable with PyInstaller\n**NOTE: This section is obsolete for now, as we are currently distributing with pip.**\n* Always run `pyinstaller rkviewer.spec` when `rkviewer.spec` is present.\n* If somehow `rkviewer.spec` went missing or you want to regenerate the build specs, run `pyinstaller -F --windowed --add-data ext/Iodine.dll;. main.py` on Windows or `pyinstaller -F -- windowed --add-data ext/Iodine.dll:. main.py` on Linux/Mac to generate a file named `main.spec`. Note that if a `main.spec` file is already  present **it will be overwritten**.\n\n## Development for Different Platforms\n\nThe python version for development was 3.7.7.\n\n### Mac Notes\n* Note that on MacOS, if you wish to use SBcoyote in a virtual environment, use `venv` instead of\n`virtualenv`, due to the latter\'s issues with wxPython.\n* pyinstaller and wxPython require a python built with `enable-framework` on. Therefore, one should do `env PYTHON_CONFIGURE_OPTS="--enable-framework" pyenv install 3.7.7` and\nuse that Python installation for building.\n* If the text is blurry in the app bundled by `pyinstaller`, one needs to add an entry in the pyinstaller settings as described [here](https://stackoverflow.com/a/40676321).\n\n### Linux Notes\n* To install wxPython on linux, see https://wxpython.org/blog/2017-08-17-builds-for-linux-with-pip/index.html. `requirements-dev.txt` and `requirements.txt` assume the user is on Ubuntu 18.04 for readthedocs. If you have a different distro and have trouble using `requirements.txt`, just install wxPython manually using the previous link.\n* Related to the last note, if readthedocs start having trouble building wxPython, understand that it might be because readthedocs updated its distro from Ubuntu 18.04. Go to `requirements-dev.txt` and change the line above `wxPython` to look in the appropriate link.\n* i.e. `-f https://extras.wxpython.org/wxPython4/extras/linux/gtk3/ubuntu-18.04/ \\n wxPython==4.1.1`\n\n## Future Development\n\n### Testing and Profiling\n* To run all tests, go to project root and run `python -m unittest discover`.\n* To run a specific test suite, run e.g. `python -m unittest test.api.test_node`.\n* Or even more specific: `python -m unittest test.api.test_node.TestNode.test_add_nodes`.\n* To profile the application, run `python -m cProfile -o rkviewer.stat main.py`.\n* To visualize the profile result, run `tuna rkviewer.stat`.\n\n### Building Local Docs\n* Run `sphinx-apidoc -f -o docs/source/rkviewer rkviewer rkviewer/plugin rkviewer/resources ` to regenerate the full reference doc source\ncode, if new files were added to the package rkviewer.\n* Run `sphinx-build -b html docs\\source docs\\build`.\n\n### Note on Style\nUsually snake_case is used for function names. However, to retain some degree of backwards\ncompatibility for wxPython, subclasses of wxPython classes use PascalCase for their methods, e.g. `Canvas::RegisterAllChildren`.\n\n### TODOs\n* ENHANCEMENT: Add support for multiple net IDs. Currently all net IDs are set to 0 by default.\n\n### Shapes TODOs\n* Events (NodeModified)\n\n### Roadmap for Shape Engine\nA shape "engine" allows the user to specify custom composite shapes for nodes and compartments.\nComposite shapes are constructed out of primitives such as circles, (rounded) rectangles, polygons,\netc.\n\nRKViewer provides a default list of (composite) shapes, but the user may also create their own\nshapes out of primitives. A (composite) shape is formed out of one or many primitives, each\nscaled, rotated, and translated by certain amounts. User-created shapes will be\nassociated with each model in the exported `.json` files.\n\nA shape-creation plugin may be created in the future to facilitate the process of designing\ncomplex shapes.\n\nHere is the roadmap for the shape engine:\n* Create preliminary list of primitives and a default list of shapes. Allow model loader/saver to\nreference that list.\n* Modify renderer to be able to render these default shapes.\n* Modify inspector to allow the user to change the properties of the primitives in the shape, such\nas colors, border thickness, etc.\n* Modify model loader/saver to allow users to create custom shape lists manually.\n* Write shape-creation plugin?\n',
    'author': 'Jin Xu and Gary Geng et al',
    'author_email': 'jxu2019@uw.edu',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7.1,<3.11',
}


setup(**setup_kwargs)
